@@ -2,7 +2,7 @@ use crate::{
     components::{
         ChangesComponent, CommandBlocking, CommandInfo,
         CommitComponent, Component, DiffComponent, DrawableComponent,
-        EventUpdate, HelpComponent, ResetComponent,
+        EventUpdate, HelpComponent, MsgComponent, ResetComponent,
     },
     keys,
     queue::{InternalEvent, Queue},
@@ -52,6 +52,7 @@ pub struct App {
     index: ChangesComponent,
     index_wd: ChangesComponent,
     diff: DiffComponent,
+    msg: MsgComponent,
     git_diff: AsyncDiff,
     git_status: AsyncStatus,
     current_commands: Vec<CommandInfo>,
@@ -68,7 +69,7 @@ impl App {
             diff_target: DiffTarget::WorkingDir,
             do_quit: false,
             reset: ResetComponent::new(queue.clone()),
-            commit: CommitComponent::default(),
+            commit: CommitComponent::new(queue.clone()),
             help: HelpComponent::default(),
             index_wd: ChangesComponent::new(
                 strings::TITLE_STATUS,
@@ -82,7 +83,8 @@ impl App {
                 false,
                 queue.clone(),
             ),
-            diff: DiffComponent::default(),
+            diff: DiffComponent::new(queue.clone()),
+            msg: MsgComponent::default(),
             git_diff: AsyncDiff::new(sender.clone()),
             git_status: AsyncStatus::new(sender),
             current_commands: Vec::new(),
@@ -152,8 +154,9 @@ impl App {
         );
 
         self.commit.draw(f, f.size());
-        self.help.draw(f, f.size());
         self.reset.draw(f, f.size());
+        self.help.draw(f, f.size());
+        self.msg.draw(f, f.size());
     }
 
     ///
@@ -216,13 +219,7 @@ impl App {
 // private impls
 impl App {
     fn update_diff(&mut self) {
-        let (idx, is_stage) = match self.diff_target {
-            DiffTarget::Stage => (&self.index, true),
-            DiffTarget::WorkingDir => (&self.index_wd, false),
-        };
-
-        if let Some(i) = idx.selection() {
-            let path = i.path;
+        if let Some((path, is_stage)) = self.selected_path() {
             let diff_params = DiffParams(path.clone(), is_stage);
 
             if self.diff.current() == (path.clone(), is_stage) {
@@ -245,6 +242,19 @@ impl App {
         }
     }
 
+    fn selected_path(&self) -> Option<(String, bool)> {
+        let (idx, is_stage) = match self.diff_target {
+            DiffTarget::Stage => (&self.index, true),
+            DiffTarget::WorkingDir => (&self.index_wd, false),
+        };
+
+        if let Some(i) = idx.selection() {
+            Some((i.path, is_stage))
+        } else {
+            None
+        }
+    }
+
     fn update_commands(&mut self) {
         self.help.set_cmds(self.commands(true));
         self.current_commands = self.commands(false);
@@ -284,6 +294,21 @@ impl App {
                 self.reset.open_for_path(p);
                 self.update_commands();
             }
+            InternalEvent::AddHunk(hash) => {
+                if let Some((path, is_stage)) = self.selected_path() {
+                    if is_stage {
+                        if sync::unstage_hunk(CWD, path, *hash) {
+                            self.update();
+                        }
+                    } else if sync::stage_hunk(CWD, path, *hash) {
+                        self.update();
+                    }
+                }
+            }
+            InternalEvent::ShowMsg(msg) => {
+                self.msg.show_msg(msg);
+                self.update();
+            }
         };
     }
 
@@ -299,9 +324,11 @@ impl App {
             }
         }
 
+        //TODO: use a new popups_list call for this
         let main_cmds_available = !self.commit.is_visible()
             && !self.help.is_visible()
-            && !self.reset.is_visible();
+            && !self.reset.is_visible()
+            && !self.msg.is_visible();
 
         {
             {
@@ -357,6 +384,7 @@ impl App {
 
     fn components(&self) -> Vec<&dyn Component> {
         vec![
+            &self.msg,
             &self.reset,
             &self.commit,
             &self.help,
@@ -368,6 +396,7 @@ impl App {
 
     fn components_mut(&mut self) -> Vec<&mut dyn Component> {
         vec![
+            &mut self.msg,
             &mut self.reset,
             &mut self.commit,
             &mut self.help,
@@ -450,6 +479,7 @@ impl App {
             };
 
             self.update_diff();
+            self.update_commands();
         }
     }
 
