@@ -6,6 +6,10 @@ pub enum InternalEvent {
     ConfirmResetFile(String),
     ///
     ResetFile(String),
+    ///
+    AddHunk(u64),
+    ///
+    ShowMsg(String),
 }
 
 ///
